/***********************************************************************************************************************
* File Name    : r_IIC_LIB.c
* Version      : 1.01
* program      : `R
* Device(s)    : R5F10Y46
* Tool-Chain   : CCRL
* Description  : This file implements IIC control functions.
* Creation Date: 2015/09/14
* Modified Date: 2015/10/06 unify inteface
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "r_IIC_LIB.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
#pragma interrupt INTIIC00 r_iic00_interrupt
#pragma interrupt INTTM01 r_inttm01_interrupt

/***********************************************************************************************************************
Global variables
***********************************************************************************************************************/

volatile uint8_t   g_iic00_status;              /* iic00 master operationflag   */
/*  00 : operation end witout error
    01 : data transmit mode
    41 : data receive mode
    81 : slave address transmit for receive mode
    FF : operation error
*/
volatile uint8_t * gp_iic00_rx_address; /* iic00 receive buffer address */
volatile uint16_t  g_iic00_rx_len;      /* iic00 receive data length    */
volatile uint16_t  g_iic00_rx_cnt;      /* iic00 receive data count     */
volatile uint8_t * gp_iic00_tx_address; /* iic00 send buffer address    */
volatile uint16_t  g_iic00_tx_cnt;      /* iic00 send data count        */

volatile uint8_t   g_TM01flag;          /* TM01 count end flag          */
volatile uint8_t   g_TM01cnt;           /* TM01 counter                 */

/***********************************************************************************************************************
Local functions
***********************************************************************************************************************/

void R_IIC00_start_condition(void);     /* issue START condition        */
void R_IIC00_Stop(void);                /* stop IIC00 operation         */

/*******************************************************************************
* Function Name: IIC_TM01_init
* Description  : disable external edge and start TM01 and enable IIC00 interrupt
* Arguments    : none
* Return Value : none
********************************************************************************/
void IIC_TM01_init(void)
{
    TMR01H = 0x00;                      /* disable edge detection   */
    TMR01L = 0x09;                      /* enable re_trigger        */
    TS0 = 0x02;                         /* enable TM01              */
    TMMK01 = 1;                         /* mask TM01 interrupt      */
    g_TM01flag = 0;                     /* clear TM01 flag          */
    g_TM01cnt = 0;                      /* clear 5us counter        */
    IICMK00 = 0;                        /* enable IIC00 interrupt   */
}
/*******************************************************************************
* Function Name: wait_time
* Description  : This function wait time
* Arguments    : wait time by 20us
* Return Value : none
********************************************************************************/
void wait_time(uint8_t time)
{
    uint8_t work;
    for ( work = time ; work > 0 ; work-- )
    {
        wait_20us();                    /* wait 20us                    */
    }
}

/*******************************************************************************
* Function Name: wait_20us
* Description  : This function wait time
* Arguments    : none
* Return Value : none
********************************************************************************/
void wait_20us(void)
{
    uint8_t CNT;
    for ( CNT = 4 ; CNT > 0 ; CNT-- )
    {
        TMIF01 = 0;                     /* clear INTTM01 request flag   */
        TS0 = 0x02;                     /* trigger TM01 count           */
        while ( TMIF01 ==0 )            /* wait for 5us delay time      */
        {
            NOP();
        }

    }
}

/*******************************************************************************
* Function Name: wait_5us
* Description  : This function wait 5us
* Arguments    : none
* Return Value : none
********************************************************************************/
void wait_5us(void)
{

    TMIF01 = 0;
    TS0 = 0x02;                         /* trigger TM01                 */
    while ( TMIF01 == 0 )               /* wait for TM01 count up       */
    {
        NOP();
    }
}

/*******************************************************************************
* Function Name: R_IIC_Master_Send
* Description  : This function starts IIC00 in transmision mode.
* Arguments    : adr -
*                    slave address
*                tx_buf -
*                    transfer data pointer
*                tx_num -
*                    transmit data number
* Return Value : MD_OK
********************************************************************************/
MD_STATUS R_IIC_Master_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{

    /* copy transfer parameters */
    g_iic00_tx_cnt = tx_num;            /* set data number to transmit  */
    gp_iic00_tx_address = tx_buf;       /* set data pointer             */

    /* start to access IIC bus  */

    R_IIC00_start_condition();          /* issue start condition        */

    g_iic00_status = 0x01;              /* set execution flag           */

    SIO00 = adr & 0xFE;                 /* transmit slave address       */
    g_TM01flag = 0;                     /* clear enable next operation  */

    return( MD_OK );

}

/*******************************************************************************
* Function Name: R_IIC_Master_Receive
* Description  : This function starts IIC00 in receive mode.
* Arguments    : adr -
*                    slave address
*                rx_buf -
*                    received data buffer pointer
*                rx_num -
*                    receive data size+1
* Return Value : MD_OK
********************************************************************************/
MD_STATUS R_IIC_Master_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{

    /* copy parameters  */

    g_iic00_rx_len = rx_num;            /* set data number to transmit  */
    g_iic00_rx_cnt = 0U;
    gp_iic00_rx_address = rx_buf;       /* set data pointer             */

    /* start to access IIC bus  */

    R_IIC00_start_condition();          /* issue start condition        */

    g_iic00_status = 0x81;              /* set execution flag           */

    SIO00 = adr | 0x01;                 /* transmit slave address       */
    g_TM01flag = 0;                     /* clear enable next operation  */

    return( MD_OK );

}

/*******************************************************************************
* Function Name: R_IIC_check_comstate
* Description  : This function get communication status
* Arguments    : none
* Return Value : result status
                0x00 : success
                0x01 : transmit operating
                0x41 : receive operating
                0x81 : transmit slave address for receive
                0xFF : error
********************************************************************************/
MD_STATUS R_IIC_check_comstate(void)
{

    return( g_iic00_status );

}

/*******************************************************************************
* Function Name: R_IIC_wait_comend
* Description  : This function wait for communication end
* Arguments    : none
* Return Value : result status
                0x00 : success
                other : error
********************************************************************************/
MD_STATUS R_IIC_wait_comend(void)
{
    MD_STATUS status;

    while( (g_iic00_status & 0x3F) == 0x01 )
    {
        HALT();                         /* wait for communication end   */
    }

    status = g_iic00_status;            /* set result status            */

    if ( status == 0xFF )
    {
        R_IIC_StopCondition();          /* issue STOP conditiopn        */
        wait_time(3);                   /* wait for 60us                */
    }

    return( status );
}

/*******************************************************************************
* Function Name: R_IIC_StopCondition
* Description  : This function issue start condition
* Arguments    : none
* Return Value : MD_OK
********************************************************************************/
MD_STATUS R_IIC_StopCondition(void)
{

    wait_time(1);                       /* dummy wait 20us              */

    R_IIC00_Stop();                     /* stop IIC00 operation         */

    SOE0 &= ~0x01;                      /* disable IIC00 output         */
    SO0 &= ~0x01;                       /* fall down SDA signal         */

    wait_5us();                         /* for command interval         */

    CKO0 |= 0x01;                       /* rise up SCL signal           */

    wait_5us();                         /* for command interval         */

    SO0 |= 0x01;                        /* rise up SDA signal           */

    wait_time(1);

    SCR00H &= ~0xC0;                    /* clear TX and RX enable        */

    g_TM01flag = 0;

    return( MD_OK );

}

/*******************************************************************************
* Function Name: R_IIC00_start_condition
* Description  : This function issue start condition
* Arguments    : none
* Return Value : none
********************************************************************************/
void R_IIC00_start_condition(void)
{

    R_IIC00_Stop();                     /* stop IIC00 operation         */

    wait_5us();                         /* wait for bus free time       */

    if ( SO0 & 0x01 == 0 )
    {
        SO0 |= 0x01;                    /* rise SO00 bit                */
        wait_5us();                     /* wait for bus free time       */
    }

    if ( CKO0 & 0x01 == 0 )
    {
        CKO0 |= 0x01;                   /* rise up SCL signal           */
        wait_5us();
    }
    
    SO0 &= ~0x01;                       /* fall down SDA when in SCL=H  */

    wait_5us();                         /* wait for bus setup time      */

    wait_time(2);                       /* wait for hold time  40us     */

    CKO0 &= ~0x01;                      /* fall down SCL after SDA =L   */

    SOE0 |= 0x01;                       /* enable channel00 output      */
    SCR00H |= 0x80;                     /* set transmit enable          */
    SS0 = 0x01;                         /* enable channel00 operation   */

    wait_5us();                         /* for command interval         */

    wait_time(1);                       /* wait for 20us to next        */

}

/*******************************************************************************
* Function Name: R_IIC00_Stop
* Description  : This function stop IIC00 operation
* Arguments    : none
* Return Value : none
********************************************************************************/
void R_IIC00_Stop(void)
{

    uint8_t work;

    wait_time(1);                       /* dummy wait 20us              */

    work = 0x40 & SSR00;
    if ( work != 0 )
    {
        while ( work != 0)              /* wait for previous execution  */
        {
            work = SSR00;               /* get IIC00 status             */
            work &= 0x40;               /* get transfer status (TSF)    */
        }
        wait_5us();
    }

    ST0 = 0x01;                         /* stop channel 0 operation     */

}

/********************************************************************************
* Function Name: r_iic00_interrupt
* Description  : This function is IIC00 interrupt service routine.
                ( only start TM03 operation. )
* Arguments    : None
* Return Value : None
*********************************************************************************/
__interrupt static void r_iic00_interrupt(void)
{

    TMIF01 = 0;                         /* clear TM01 interrupt flag    */
    TS0 = 0x02;                         /* start TM01                   */
    g_TM01cnt = 4-1;                    /* set 5us * 4 = 20us           */
    TMMK01 = 0;                         /* enable TM01 vector interrupt */
    g_TM01flag = 1;
}



/********************************************************************************
* Function Name: r_inttm01_interrupt
* Description  : This function is TM01 interrupt service routine.
                ( actual procedure of IIC01 interrupt. )
* Arguments    : None
* Return Value : None
*********************************************************************************/
__interrupt static void r_inttm01_interrupt(void)
{

    uint8_t work;                       /* work register            */

  if ( g_TM01cnt == 0 )
    {
        if( g_TM01flag == 1 )
        {
            g_TM01flag = 0;             /* clear TM01 count flag    */

            TMMK01 =1;                  /* mask TM01 interrupt      */

            if ( ( g_iic00_status & 0x7F ) == 0x01 )
            {                           /* IIC00 is working         */

/*--------------------------------------------------------------
        IIC00 data transfer end

    check ACK from slave 

----------------------------------------------------------------*/

                work = SSR00;           /* read result falg         */
                SIR00 = work;           /* clear error flag         */

                if( work & 0x02 )

/*  NACK from slave          */

                {
                    g_iic00_status = 0xFF; /* set error flag        */
                }
                else                    /* ACK from slave           */
                {
                    if( g_iic00_status == 0x01 ) /* transmit data operation */
                    {
/*--------------------------------------------------------------
        data is transmitted 
----------------------------------------------------------------*/

                        if ( g_iic00_tx_cnt > 0 ) /* more data to transmit */
                        {               /* set transmit data        */
                            SIO00 = *gp_iic00_tx_address;
                            gp_iic00_tx_address++;
                            g_iic00_tx_cnt--;
                        }
                        else            /* data transmit complete   */
                        {
                            g_iic00_status = 0; /* set data transfer end    */
                        }
                    }
                    else 

/*===================================================================

    slave address for receive operation

    g_iic00_status = 0x81 : slave address is transmitted

=====================================================================*/
                    {

/*-------------------------------------------------------------------
    slave address end and start receive
---------------------------------------------------------------------*/
                        ST0 = 0x01;     /* stop IIC00 operaion      */
                        SCR00H ^= 0xC0; /* TX disable & enable RX   */
                        SS0 = 0x01;     /* start IIC00 operation    */
                        SIO00 = 0xFF;   /* trigger RX operation     */
                        g_iic00_status = 0x41; /* change status     */
                    }
                }
            }
            else
            {
                if ( g_iic00_status == 0x41 )
                {
/*--------------------------------------------------------------
        data receive operation

    store received data
----------------------------------------------------------------*/
                    *gp_iic00_rx_address = SIO00;

                    if ( g_iic00_rx_cnt < g_iic00_rx_len )
                    {
/*-------------------------------------------------------------------
        more data to receive is exist

    change parameters
---------------------------------------------------------------------*/

                        gp_iic00_rx_address++;
                        g_iic00_rx_cnt++;

                        if ( g_iic00_rx_cnt == g_iic00_rx_len )
                        {
/*-------------------------------------------------------------------
    next data is last data to receive. then NACK response
---------------------------------------------------------------------*/

                            SOE0 &= ~0x01;
                        }
/* start to receive next data   */
                        SIO00 = 0xFF;
                    }
                    else                /* data receive complete    */
                    {
/*-------------------------------------------------------------------
    complete receive data
---------------------------------------------------------------------*/
                        g_iic00_status = 0x00;  /* set success flag */
                    }
                }
            }
        }
    }
    else
    {
        g_TM01cnt--;
        TS0 = 0x02;                     /* start TM01                   */
    }

}



